@@ -25,7 +25,6 @@
 #include "librpc/gen_ndr/samr.h" /* for struct samrPassword */
 #include "auth/credentials/credentials.h"
 #include "auth/credentials/credentials_internal.h"
-#include "auth/gensec/gensec.h"
 #include "libcli/auth/libcli_auth.h"
 #include "tevent.h"
 #include "param/param.h"
@@ -130,21 +129,6 @@ _PUBLIC_ void *_cli_credentials_callback_data(struct cli_credentials *cred)
 	return cred->priv_data;
 }
 
-_PUBLIC_ struct cli_credentials *cli_credentials_shallow_copy(TALLOC_CTX *mem_ctx,
-						struct cli_credentials *src)
-{
-	struct cli_credentials *dst;
-
-	dst = talloc(mem_ctx, struct cli_credentials);
-	if (dst == NULL) {
-		return NULL;
-	}
-
-	*dst = *src;
-
-	return dst;
-}
-
 /**
  * Create a new anonymous credential
  * @param mem_ctx TALLOC_CTX parent for credentials structure 
@@ -363,8 +347,6 @@ _PUBLIC_ bool cli_credentials_set_principal_callback(struct cli_credentials *cre
 
 _PUBLIC_ bool cli_credentials_authentication_requested(struct cli_credentials *cred) 
 {
-	uint32_t gensec_features = 0;
-
 	if (cred->bind_dn) {
 		return true;
 	}
@@ -392,19 +374,6 @@ _PUBLIC_ bool cli_credentials_authentication_requested(struct cli_credentials *c
 		return true;
 	}
 
-	gensec_features = cli_credentials_get_gensec_features(cred);
-	if (gensec_features & GENSEC_FEATURE_NTLM_CCACHE) {
-		return true;
-	}
-
-	if (gensec_features & GENSEC_FEATURE_SIGN) {
-		return true;
-	}
-
-	if (gensec_features & GENSEC_FEATURE_SEAL) {
-		return true;
-	}
-
 	return false;
 }
 
