@@ -24,6 +24,10 @@
 #include "lib/socket/netif.h"
 
 void init_glue(void);
+static PyObject *PyExc_NTSTATUSError;
+static PyObject *PyExc_WERRORError;
+static PyObject *PyExc_HRESULTError;
+static PyObject *PyExc_DsExtendedError;
 
 static PyObject *py_generate_random_str(PyObject *self, PyObject *args)
 {
@@ -121,6 +125,15 @@ static PyObject *py_get_debug_level(PyObject *self)
 	return PyInt_FromLong(DEBUGLEVEL);
 }
 
+static PyObject *py_is_ntvfs_fileserver_built(PyObject *self)
+{
+#ifdef WITH_NTVFS_FILESERVER
+	Py_RETURN_TRUE;
+#else
+	Py_RETURN_FALSE;
+#endif
+}
+
 /*
   return the list of interface IPs we have configured
   takes an loadparm context, returns a list of IPs in string form
@@ -267,6 +280,8 @@ static PyMethodDef py_misc_methods[] = {
 		"(for testing) compare two strings using Samba's strcasecmp_m()"},
 	{ "strstr_m", (PyCFunction)py_strstr_m, METH_VARARGS,
 		"(for testing) find one string in another with Samba's strstr_m()"},
+	{ "is_ntvfs_fileserver_built", (PyCFunction)py_is_ntvfs_fileserver_built, METH_NOARGS,
+		"is the NTVFS file server built in this installation?" },
 	{ NULL }
 };
 
@@ -283,5 +298,29 @@ void init_glue(void)
 
 	PyModule_AddObject(m, "version",
 					   PyString_FromString(SAMBA_VERSION_STRING));
+	PyExc_NTSTATUSError = PyErr_NewException(discard_const_p(char, "samba.NTSTATUSError"), PyExc_RuntimeError, NULL);
+	if (PyExc_NTSTATUSError != NULL) {
+		Py_INCREF(PyExc_NTSTATUSError);
+		PyModule_AddObject(m, "NTSTATUSError", PyExc_NTSTATUSError);
+	}
+
+	PyExc_WERRORError = PyErr_NewException(discard_const_p(char, "samba.WERRORError"), PyExc_RuntimeError, NULL);
+	if (PyExc_WERRORError != NULL) {
+		Py_INCREF(PyExc_WERRORError);
+		PyModule_AddObject(m, "WERRORError", PyExc_WERRORError);
+	}
+
+	PyExc_HRESULTError = PyErr_NewException(discard_const_p(char, "samba.HRESULTError"), PyExc_RuntimeError, NULL);
+	if (PyExc_HRESULTError != NULL) {
+		Py_INCREF(PyExc_HRESULTError);
+		PyModule_AddObject(m, "HRESULTError", PyExc_HRESULTError);
+	}
+
+	PyExc_DsExtendedError = PyErr_NewException(discard_const_p(char, "samba.DsExtendedError"), PyExc_RuntimeError, NULL);
+	if (PyExc_DsExtendedError != NULL) {
+		Py_INCREF(PyExc_DsExtendedError);
+		PyModule_AddObject(m, "DsExtendedError", PyExc_DsExtendedError);
+	}
+
 }
 
