@@ -33,7 +33,7 @@
 
 struct perm_value {
 	const char *perm;
-	uint32 mask;
+	uint32_t mask;
 };
 
 static const struct perm_value special_values[] = {
@@ -84,7 +84,7 @@ static NTSTATUS cli_lsa_lookup_sid(struct cli_state *cli,
 				   enum lsa_SidType *type,
 				   char **domain, char **name)
 {
-	uint16 orig_cnum = cli_state_get_tid(cli);
+	uint16_t orig_cnum = cli_state_get_tid(cli);
 	struct rpc_pipe_client *p = NULL;
 	struct policy_handle handle;
 	NTSTATUS status;
@@ -165,7 +165,7 @@ static NTSTATUS cli_lsa_lookup_name(struct cli_state *cli,
 				    enum lsa_SidType *type,
 				    struct dom_sid *sid)
 {
-	uint16 orig_cnum = cli_state_get_tid(cli);
+	uint16_t orig_cnum = cli_state_get_tid(cli);
 	struct rpc_pipe_client *p;
 	struct policy_handle handle;
 	NTSTATUS status;
@@ -292,7 +292,7 @@ void print_ace(struct cli_state *cli, FILE *f, struct security_ace *ace,
 	const struct perm_value *v;
 	fstring sidstr;
 	int do_print = 0;
-	uint32 got_mask;
+	uint32_t got_mask;
 
 	SidToString(cli, sidstr, &ace->trustee, numeric);
 
@@ -418,14 +418,6 @@ bool parse_ace(struct cli_state *cli, struct security_ace *ace,
 	}
 	*p = '\0';
 	p++;
-	/* Try to parse numeric form */
-
-	if (sscanf(p, "%u/%u/%u", &atype, &aflags, &amask) == 3 &&
-	    StringToSid(cli, &sid, str)) {
-		goto done;
-	}
-
-	/* Try to parse text form */
 
 	if (!StringToSid(cli, &sid, str)) {
 		printf("ACE '%s': failed to convert '%s' to SID\n",
@@ -448,6 +440,33 @@ bool parse_ace(struct cli_state *cli, struct security_ace *ace,
 		atype = SEC_ACE_TYPE_ACCESS_ALLOWED;
 	} else if (strncmp(tok, "DENIED", strlen("DENIED")) == 0) {
 		atype = SEC_ACE_TYPE_ACCESS_DENIED;
+
+	} else if (strnequal(tok, "0x", 2)) {
+		int result;
+
+		result = sscanf(tok, "%x", &atype);
+		if (result == 0 ||
+		    (atype != SEC_ACE_TYPE_ACCESS_ALLOWED &&
+		     atype != SEC_ACE_TYPE_ACCESS_DENIED)) {
+			printf("ACE '%s': bad hex value for type at '%s'\n",
+			       orig_str, tok);
+			SAFE_FREE(str);
+			TALLOC_FREE(frame);
+			return false;
+		}
+	} else if(tok[0] >= '0' && tok[0] <= '9') {
+		int result;
+
+		result = sscanf(tok, "%u", &atype);
+		if (result == 0 ||
+		    (atype != SEC_ACE_TYPE_ACCESS_ALLOWED &&
+		     atype != SEC_ACE_TYPE_ACCESS_DENIED)) {
+			printf("ACE '%s': bad integer value for type at '%s'\n",
+			       orig_str, tok);
+			SAFE_FREE(str);
+			TALLOC_FREE(frame);
+			return false;
+		}
 	} else {
 		printf("ACE '%s': missing 'ALLOWED' or 'DENIED' entry at '%s'\n",
 			orig_str, tok);
@@ -456,8 +475,6 @@ bool parse_ace(struct cli_state *cli, struct security_ace *ace,
 		return False;
 	}
 
-	/* Only numeric form accepted for flags at present */
-
 	if (!next_token_talloc(frame, &cp, &tok, "/")) {
 		printf("ACE '%s': bad flags entry at '%s'\n",
 			orig_str, tok);
@@ -580,7 +597,7 @@ void sec_desc_print(struct cli_state *cli, FILE *f,
 		    struct security_descriptor *sd, bool numeric)
 {
 	fstring sidstr;
-	uint32 i;
+	uint32_t i;
 
 	fprintf(f, "REVISION:%d\n", sd->revision);
 	print_acl_ctrl(f, sd->type, numeric);
