@@ -417,7 +417,7 @@ static struct SMB4ACL_T *nfs4acls_inheritacl(vfs_handle_struct *handle,
 	TALLOC_CTX *frame = talloc_stackframe();
 
 	DEBUG(10, ("nfs4acls_inheritacl invoked for %s\n", path));
-	smb_fname = synthetic_smb_fname(frame, path, NULL, NULL);
+	smb_fname = synthetic_smb_fname(frame, path, NULL, NULL, 0);
 	if (smb_fname == NULL) {
 		TALLOC_FREE(frame);
 		errno = ENOMEM;
@@ -541,12 +541,14 @@ static NTSTATUS nfs4acl_xattr_fget_nt_acl(struct vfs_handle_struct *handle,
 }
 
 static NTSTATUS nfs4acl_xattr_get_nt_acl(struct vfs_handle_struct *handle,
-				  const char *name, uint32_t security_info,
+				  const struct smb_filename *smb_fname,
+				  uint32_t security_info,
 				  TALLOC_CTX *mem_ctx,
 				  struct security_descriptor **ppdesc)
 {
 	struct SMB4ACL_T *pacl;
 	NTSTATUS status;
+	const char *name = smb_fname->base_name;
 	TALLOC_CTX *frame = talloc_stackframe();
 
 	status = nfs4_get_nfs4_acl(handle, frame, name, &pacl);
@@ -558,7 +560,7 @@ static NTSTATUS nfs4acl_xattr_get_nt_acl(struct vfs_handle_struct *handle,
 		return status;
 	}
 
-	status = smb_get_nt_acl_nfs4(handle->conn, name, security_info,
+	status = smb_get_nt_acl_nfs4(handle->conn, smb_fname, security_info,
 				     mem_ctx, ppdesc,
 				     pacl);
 	TALLOC_FREE(frame);
