@@ -33,9 +33,13 @@
 #include "printing/nt_printing_migrate.h"
 
 #define FORMS_PREFIX "FORMS/"
+#define FORMS_PREFIX_LEN 6
 #define DRIVERS_PREFIX "DRIVERS/"
+#define DRIVERS_PREFIX_LEN 8
 #define PRINTERS_PREFIX "PRINTERS/"
+#define PRINTERS_PREFIX_LEN 9
 #define SECDESC_PREFIX "SECDESC/"
+#define SECDESC_PREFIX_LEN 8
 
 #define ARG_ENCODING "encoding="
 
@@ -216,7 +220,7 @@ static int net_printing_dump(struct net_context *c, int argc,
 	int ret = -1;
 	TALLOC_CTX *ctx = talloc_stackframe();
 	TDB_CONTEXT *tdb;
-	TDB_DATA kbuf, dbuf;
+	TDB_DATA kbuf, newkey, dbuf;
 	struct printing_opts *o;
 	const char *save_dos_charset = lp_dos_charset();
 	bool do_string_conversion = false;
@@ -254,42 +258,95 @@ static int net_printing_dump(struct net_context *c, int argc,
 		do_string_conversion = true;
 	}
 
-	for (kbuf = tdb_firstkey_compat(tdb);
+	for (kbuf = tdb_firstkey(tdb);
 	     kbuf.dptr;
-	     kbuf = tdb_nextkey_compat(tdb, kbuf))
+	     newkey = tdb_nextkey(tdb, kbuf), free(kbuf.dptr), kbuf=newkey)
 	{
-		dbuf = tdb_fetch_compat(tdb, kbuf);
+		int cmp;
+
+		dbuf = tdb_fetch(tdb, kbuf);
 		if (!dbuf.dptr) {
 			continue;
 		}
 
-		if (strncmp((const char *)kbuf.dptr, FORMS_PREFIX, strlen(FORMS_PREFIX)) == 0) {
-			dump_form(ctx, (const char *)kbuf.dptr+strlen(FORMS_PREFIX), dbuf.dptr, dbuf.dsize);
+		cmp = strncmp((const char *)kbuf.dptr,
+			      FORMS_PREFIX,
+			      FORMS_PREFIX_LEN);
+		if (cmp == 0) {
+			char *key_name = NULL;
+			size_t converted_size = 0;
+			bool ok;
+
+			ok = pull_ascii_talloc(ctx,
+					       &key_name,
+					       (const char *) kbuf.dptr + strlen(FORMS_PREFIX),
+					       &converted_size);
+			if (!ok) {
+				continue;
+			}
+
+			dump_form(ctx, key_name, dbuf.dptr, dbuf.dsize);
+			TALLOC_FREE(key_name);
 			SAFE_FREE(dbuf.dptr);
 			continue;
 		}
 
-		if (strncmp((const char *)kbuf.dptr, DRIVERS_PREFIX, strlen(DRIVERS_PREFIX)) == 0) {
+		cmp = strncmp((const char *)kbuf.dptr,
+			      DRIVERS_PREFIX,
+			      DRIVERS_PREFIX_LEN);
+		if (cmp == 0) {
+			char *key_name = NULL;
+			size_t converted_size = 0;
+			bool ok;
+
+			ok = pull_ascii_talloc(ctx,
+					       &key_name,
+					       (const char *) kbuf.dptr + strlen(DRIVERS_PREFIX),
+					       &converted_size);
+			if (!ok) {
+				continue;
+			}
+
 			dump_driver(ctx,
-				    (const char *)kbuf.dptr+strlen(DRIVERS_PREFIX),
+				    key_name,
 				    dbuf.dptr,
 				    dbuf.dsize,
 				    do_string_conversion);
+			TALLOC_FREE(key_name);
 			SAFE_FREE(dbuf.dptr);
 			continue;
 		}
 
-		if (strncmp((const char *)kbuf.dptr, PRINTERS_PREFIX, strlen(PRINTERS_PREFIX)) == 0) {
+		cmp = strncmp((const char *)kbuf.dptr,
+			      PRINTERS_PREFIX,
+			      PRINTERS_PREFIX_LEN);
+		if (cmp == 0) {
+			char *key_name = NULL;
+			size_t converted_size = 0;
+			bool ok;
+
+			ok = pull_ascii_talloc(ctx,
+					       &key_name,
+					       (const char *) kbuf.dptr + strlen(PRINTERS_PREFIX),
+					       &converted_size);
+			if (!ok) {
+				continue;
+			}
+
 			dump_printer(ctx,
-				     (const char *)kbuf.dptr+strlen(PRINTERS_PREFIX),
+				     key_name,
 				     dbuf.dptr,
 				     dbuf.dsize,
 				     do_string_conversion);
+			TALLOC_FREE(key_name);
 			SAFE_FREE(dbuf.dptr);
 			continue;
 		}
 
-		if (strncmp((const char *)kbuf.dptr, SECDESC_PREFIX, strlen(SECDESC_PREFIX)) == 0) {
+		cmp = strncmp((const char *)kbuf.dptr,
+			      SECDESC_PREFIX,
+			      SECDESC_PREFIX_LEN);
+		if (cmp == 0) {
 			dump_sd(ctx, (const char *)kbuf.dptr+strlen(SECDESC_PREFIX), dbuf.dptr, dbuf.dsize);
 			SAFE_FREE(dbuf.dptr);
 			continue;
@@ -317,7 +374,7 @@ static NTSTATUS printing_migrate_internal(struct net_context *c,
 	struct printing_opts *o;
 	TALLOC_CTX *tmp_ctx;
 	TDB_CONTEXT *tdb;
-	TDB_DATA kbuf, dbuf;
+	TDB_DATA kbuf, newkey, dbuf;
 	NTSTATUS status;
 	const char *save_dos_charset = lp_dos_charset();
 	bool do_string_conversion = false;
@@ -347,54 +404,104 @@ static NTSTATUS printing_migrate_internal(struct net_context *c,
 		do_string_conversion = true;
 	}
 
-	for (kbuf = tdb_firstkey_compat(tdb);
+	for (kbuf = tdb_firstkey(tdb);
 	     kbuf.dptr;
-	     kbuf = tdb_nextkey_compat(tdb, kbuf))
+	     newkey = tdb_nextkey(tdb, kbuf), free(kbuf.dptr), kbuf = newkey)
 	{
-		dbuf = tdb_fetch_compat(tdb, kbuf);
+		int cmp;
+
+		dbuf = tdb_fetch(tdb, kbuf);
 		if (!dbuf.dptr) {
 			continue;
 		}
 
-		if (strncmp((const char *) kbuf.dptr, FORMS_PREFIX, strlen(FORMS_PREFIX)) == 0) {
+		cmp = strncmp((const char *) kbuf.dptr,
+			      FORMS_PREFIX,
+			      FORMS_PREFIX_LEN);
+		if (cmp == 0) {
+			char *key_name = NULL;
+			size_t converted_size = 0;
+			bool ok;
+
+			ok = pull_ascii_talloc(tmp_ctx,
+					       &key_name,
+					       (const char *) kbuf.dptr + strlen(FORMS_PREFIX),
+					       &converted_size);
+			if (!ok) {
+				continue;
+			}
+
 			printing_tdb_migrate_form(tmp_ctx,
 				     winreg_pipe,
-				     (const char *) kbuf.dptr + strlen(FORMS_PREFIX),
+				     key_name,
 				     dbuf.dptr,
 				     dbuf.dsize);
+			TALLOC_FREE(key_name);
 			SAFE_FREE(dbuf.dptr);
 			continue;
 		}
 
-		if (strncmp((const char *) kbuf.dptr, DRIVERS_PREFIX, strlen(DRIVERS_PREFIX)) == 0) {
+		cmp = strncmp((const char *) kbuf.dptr,
+			      DRIVERS_PREFIX,
+			      DRIVERS_PREFIX_LEN);
+		if (cmp == 0) {
+			char *key_name = NULL;
+			size_t converted_size = 0;
+			bool ok;
+
+			ok = pull_ascii_talloc(tmp_ctx,
+					       &key_name,
+					       (const char *) kbuf.dptr + strlen(DRIVERS_PREFIX),
+					       &converted_size);
+			if (!ok) {
+				continue;
+			}
+
 			printing_tdb_migrate_driver(tmp_ctx,
 				       winreg_pipe,
-				       (const char *) kbuf.dptr + strlen(DRIVERS_PREFIX),
+				       key_name,
 				       dbuf.dptr,
 				       dbuf.dsize,
 				       do_string_conversion);
+			TALLOC_FREE(key_name);
 			SAFE_FREE(dbuf.dptr);
 			continue;
 		}
 
-		if (strncmp((const char *) kbuf.dptr, PRINTERS_PREFIX, strlen(PRINTERS_PREFIX)) == 0) {
+		cmp = strncmp((const char *) kbuf.dptr,
+			      PRINTERS_PREFIX,
+			      PRINTERS_PREFIX_LEN);
+		if (cmp == 0) {
+			char *key_name = NULL;
+			size_t converted_size = 0;
+			bool ok;
+
+			ok = pull_ascii_talloc(tmp_ctx,
+					       &key_name,
+					       (const char *) kbuf.dptr + strlen(PRINTERS_PREFIX),
+					       &converted_size);
+			if (!ok) {
+				continue;
+			}
+
 			printing_tdb_migrate_printer(tmp_ctx,
 					winreg_pipe,
-					(const char *) kbuf.dptr + strlen(PRINTERS_PREFIX),
+					key_name,
 					dbuf.dptr,
 					dbuf.dsize,
 					do_string_conversion);
+			TALLOC_FREE(key_name);
 			SAFE_FREE(dbuf.dptr);
 			continue;
 		}
 		SAFE_FREE(dbuf.dptr);
 	}
 
-	for (kbuf = tdb_firstkey_compat(tdb);
+	for (kbuf = tdb_firstkey(tdb);
 	     kbuf.dptr;
-	     kbuf = tdb_nextkey_compat(tdb, kbuf))
+	     newkey = tdb_nextkey(tdb, kbuf), free(kbuf.dptr), kbuf = newkey)
 	{
-		dbuf = tdb_fetch_compat(tdb, kbuf);
+		dbuf = tdb_fetch(tdb, kbuf);
 		if (!dbuf.dptr) {
 			continue;
 		}
