@@ -26,7 +26,7 @@
 #include "param/param.h"
 #include "events/events.h"
 
-int main(int argc, char **argv)
+int main(int argc, const char **argv)
 {
   	int opt;
 	poptContext pc;
@@ -44,7 +44,7 @@ int main(int argc, char **argv)
 		{ NULL }
 	};
 
-	pc = poptGetContext(argv[0], argc, (const char **) argv, long_options,0);
+	pc = poptGetContext(argv[0], argc, argv, long_options,0);
 
 	while((opt = poptGetNextOpt(pc)) != -1) {
 	}
