@@ -23,6 +23,7 @@
 #include "torture/ndr/proto.h"
 #include "../lib/util/dlinklist.h"
 #include "param/param.h"
+#include "librpc/gen_ndr/ndr_misc.h"
 
 struct ndr_pull_test_data {
 	DATA_BLOB data;
@@ -31,8 +32,26 @@ struct ndr_pull_test_data {
 	ndr_pull_flags_fn_t pull_fn;
 	ndr_push_flags_fn_t push_fn;
 	int ndr_flags;
+	int flags;
 };
 
+static enum ndr_err_code torture_ndr_push_struct_blob_flags(DATA_BLOB *blob, TALLOC_CTX *mem_ctx, uint32_t flags, uint32_t ndr_flags, const void *p, ndr_push_flags_fn_t fn)
+{
+	struct ndr_push *ndr;
+	ndr = ndr_push_init_ctx(mem_ctx);
+	NDR_ERR_HAVE_NO_MEMORY(ndr);
+
+	ndr->flags |= ndr_flags;
+
+	NDR_CHECK(fn(ndr, flags, p));
+
+	*blob = ndr_push_blob(ndr);
+	talloc_steal(mem_ctx, blob->data);
+	talloc_free(ndr);
+
+	return NDR_ERR_SUCCESS;
+}
+
 static bool wrap_ndr_pullpush_test(struct torture_context *tctx,
 				   struct torture_tcase *tcase,
 				   struct torture_test *test)
@@ -42,15 +61,24 @@ static bool wrap_ndr_pullpush_test(struct torture_context *tctx,
 	struct ndr_pull *ndr = ndr_pull_init_blob(&(data->data), tctx);
 	void *ds = talloc_zero_size(ndr, data->struct_size);
 	bool ret;
+	uint32_t highest_ofs;
+
+	ndr->flags |= data->flags;
 
 	ndr->flags |= LIBNDR_FLAG_REF_ALLOC;
 
 	torture_assert_ndr_success(tctx, data->pull_fn(ndr, data->ndr_flags, ds),
 				   "pulling");
 
-	torture_assert(tctx, ndr->offset == ndr->data_size,
+	if (ndr->offset > ndr->relative_highest_offset) {
+		highest_ofs = ndr->offset;
+	} else {
+		highest_ofs = ndr->relative_highest_offset;
+	}
+
+	torture_assert(tctx, highest_ofs == ndr->data_size,
 				   talloc_asprintf(tctx,
-					   "%d unread bytes", ndr->data_size - ndr->offset));
+					   "%d unread bytes", ndr->data_size - highest_ofs));
 
 	if (check_fn != NULL) {
 		ret = check_fn(tctx, ds);
@@ -60,7 +88,7 @@ static bool wrap_ndr_pullpush_test(struct torture_context *tctx,
 
 	if (data->push_fn != NULL) {
 		DATA_BLOB outblob;
-		torture_assert_ndr_success(tctx, ndr_push_struct_blob(&outblob, ndr, ds, data->push_fn), "pushing");
+		torture_assert_ndr_success(tctx, torture_ndr_push_struct_blob_flags(&outblob, ndr, data->ndr_flags, ndr->flags, ds, data->push_fn), "pushing");
 		torture_assert_data_blob_equal(tctx, outblob, data->data, "ndr push compare");
 	}
 
@@ -76,6 +104,7 @@ _PUBLIC_ struct torture_test *_torture_suite_add_ndr_pullpush_test(
 	DATA_BLOB db,
 	size_t struct_size,
 	int ndr_flags,
+	int flags,
 	bool (*check_fn) (struct torture_context *ctx, void *data))
 {
 	struct torture_test *test;
@@ -93,6 +122,7 @@ _PUBLIC_ struct torture_test *_torture_suite_add_ndr_pullpush_test(
 	data = talloc(test, struct ndr_pull_test_data);
 	data->data = db;
 	data->ndr_flags = ndr_flags;
+	data->flags = flags;
 	data->struct_size = struct_size;
 	data->pull_fn = pull_fn;
 	data->push_fn = push_fn;
@@ -101,7 +131,7 @@ _PUBLIC_ struct torture_test *_torture_suite_add_ndr_pullpush_test(
 	test->fn = check_fn;
 	test->dangerous = false;
 
-	DLIST_ADD_END(tcase->tests, test, struct torture_test *);
+	DLIST_ADD_END(tcase->tests, test);
 
 	return test;
 }
@@ -115,6 +145,7 @@ static bool wrap_ndr_inout_pull_test(struct torture_context *tctx,
 	const struct ndr_pull_test_data *data = (const struct ndr_pull_test_data *)test->data;
 	void *ds = talloc_zero_size(tctx, data->struct_size);
 	struct ndr_pull *ndr;
+	uint32_t highest_ofs;
 
 	/* handle NDR_IN context */
 
@@ -127,8 +158,14 @@ static bool wrap_ndr_inout_pull_test(struct torture_context *tctx,
 		data->pull_fn(ndr, NDR_IN, ds),
 		"ndr pull of context failed");
 
-	torture_assert(tctx, ndr->offset == ndr->data_size,
-		talloc_asprintf(tctx, "%d unread bytes", ndr->data_size - ndr->offset));
+	if (ndr->offset > ndr->relative_highest_offset) {
+		highest_ofs = ndr->offset;
+	} else {
+		highest_ofs = ndr->relative_highest_offset;
+	}
+
+	torture_assert(tctx, highest_ofs == ndr->data_size,
+		talloc_asprintf(tctx, "%d unread bytes", ndr->data_size - highest_ofs));
 
 	talloc_free(ndr);
 
@@ -143,8 +180,14 @@ static bool wrap_ndr_inout_pull_test(struct torture_context *tctx,
 		data->pull_fn(ndr, NDR_OUT, ds),
 		"ndr pull failed");
 
-	torture_assert(tctx, ndr->offset == ndr->data_size,
-		talloc_asprintf(tctx, "%d unread bytes", ndr->data_size - ndr->offset));
+	if (ndr->offset > ndr->relative_highest_offset) {
+		highest_ofs = ndr->offset;
+	} else {
+		highest_ofs = ndr->relative_highest_offset;
+	}
+
+	torture_assert(tctx, highest_ofs == ndr->data_size,
+		talloc_asprintf(tctx, "%d unread bytes", ndr->data_size - highest_ofs));
 
 	talloc_free(ndr);
 
@@ -184,7 +227,7 @@ _PUBLIC_ struct torture_test *_torture_suite_add_ndr_pull_inout_test(
 	test->fn = check_fn;
 	test->dangerous = false;
 
-	DLIST_ADD_END(tcase->tests, test, struct torture_test *);
+	DLIST_ADD_END(tcase->tests, test);
 
 	return test;
 }
@@ -325,6 +368,106 @@ static bool test_guid_string2_valid(struct torture_context *tctx)
 	return true;
 }
 
+static bool test_guid_into_blob(struct torture_context *tctx)
+{
+	enum ndr_err_code ndr_err;
+	static const char exp_guid[16] =
+		{ 0x1, 0x0, 0x0, 0x0,
+		  0x2, 0x0, 0x3, 0x0,
+		  0x4, 0x5, 0x6, 0x7,
+		  0x8, 0x9, 0xa, 0xb };
+	DATA_BLOB exp = data_blob_const(exp_guid, 16);
+	char ndr_guid[16] =
+		{ 0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0 };
+	DATA_BLOB b = data_blob_const(ndr_guid, 16);
+	struct GUID guid;
+	guid.time_low = 1;
+	guid.time_mid = 2;
+	guid.time_hi_and_version = 3;
+	guid.clock_seq[0] = 4;
+	guid.clock_seq[1] = 5;
+	guid.node[0] = 6;
+	guid.node[1] = 7;
+	guid.node[2] = 8;
+	guid.node[3] = 9;
+	guid.node[4] = 10;
+	guid.node[5] = 11;
+	
+	ndr_err = ndr_push_struct_into_fixed_blob(&b, &guid,
+						  (ndr_push_flags_fn_t)ndr_push_GUID);
+	torture_assert_ndr_err_equal(tctx, ndr_err, NDR_ERR_SUCCESS,
+				     "wrong NDR error");
+	torture_assert_data_blob_equal(tctx, b, exp,
+				       "GUID packed wrongly");
+
+	return true;
+}
+
+/* Really a test of ndr_push_struct_into_fixed_blob error handling */
+static bool test_guid_into_long_blob(struct torture_context *tctx)
+{
+	enum ndr_err_code ndr_err;
+	char ndr_guid[17] =
+		{ 0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0, 0x0 };
+	DATA_BLOB b = data_blob_const(ndr_guid, 17);
+	struct GUID guid;
+	guid.time_low = 1;
+	guid.time_mid = 2;
+	guid.time_hi_and_version = 3;
+	guid.clock_seq[0] = 4;
+	guid.clock_seq[1] = 5;
+	guid.node[0] = 6;
+	guid.node[1] = 7;
+	guid.node[2] = 8;
+	guid.node[3] = 9;
+	guid.node[4] = 10;
+	guid.node[5] = 11;
+
+	torture_assert(tctx, b.data != NULL, "data_blob_talloc failed");
+	ndr_err = ndr_push_struct_into_fixed_blob(
+		&b, &guid, (ndr_push_flags_fn_t)ndr_push_GUID);
+	torture_assert_ndr_err_equal(tctx, ndr_err, NDR_ERR_BUFSIZE,
+				     "wrong NDR error");
+
+	return true;
+}
+
+static bool test_guid_into_short_blob(struct torture_context *tctx)
+{
+	enum ndr_err_code ndr_err;
+	char ndr_guid[15] =
+		{ 0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0, 0x0,
+		  0x0, 0x0, 0x0 };
+	DATA_BLOB b = data_blob_const(ndr_guid, 15);
+	struct GUID guid;
+	guid.time_low = 1;
+	guid.time_mid = 2;
+	guid.time_hi_and_version = 3;
+	guid.clock_seq[0] = 4;
+	guid.clock_seq[1] = 5;
+	guid.node[0] = 6;
+	guid.node[1] = 7;
+	guid.node[2] = 8;
+	guid.node[3] = 9;
+	guid.node[4] = 10;
+	guid.node[5] = 11;
+
+	ndr_err = ndr_push_struct_into_fixed_blob(
+		&b, &guid, (ndr_push_flags_fn_t)ndr_push_GUID);
+	torture_assert_ndr_err_equal(tctx, ndr_err, NDR_ERR_BUFSIZE,
+				     "wrong NDR error");
+
+	return true;
+}
+
 static bool test_compare_uuid(struct torture_context *tctx)
 {
 	struct GUID g1, g2;
@@ -373,10 +516,12 @@ struct torture_suite *torture_local_ndr(TALLOC_CTX *mem_ctx)
 	torture_suite_add_suite(suite, ndr_drsblobs_suite(suite));
 	torture_suite_add_suite(suite, ndr_nbt_suite(suite));
 	torture_suite_add_suite(suite, ndr_ntlmssp_suite(suite));
-#ifdef AD_DC_BUILD_IS_ENABLED /* Add Heimdal-specific KDC test */
 	torture_suite_add_suite(suite, ndr_backupkey_suite(suite));
-#endif
+	torture_suite_add_suite(suite, ndr_witness_suite(suite));
+	torture_suite_add_suite(suite, ndr_clusapi_suite(suite));
+	torture_suite_add_suite(suite, ndr_negoex_suite(suite));
 	torture_suite_add_suite(suite, ndr_string_suite(suite));
+	torture_suite_add_suite(suite, ndr_krb5pac_suite(suite));
 
 	torture_suite_add_simple_test(suite, "string terminator",
 				      test_check_string_terminator);
@@ -402,6 +547,15 @@ struct torture_suite *torture_local_ndr(TALLOC_CTX *mem_ctx)
 	torture_suite_add_simple_test(suite, "compare_uuid",
 				      test_compare_uuid);
 
+	torture_suite_add_simple_test(suite, "guid_into_blob",
+				      test_guid_into_blob);
+
+	torture_suite_add_simple_test(suite, "guid_into_short_blob",
+				      test_guid_into_short_blob);
+
+	torture_suite_add_simple_test(suite, "guid_into_long_blob",
+				      test_guid_into_long_blob);
+
 	return suite;
 }
 
